# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {}

modules = [].py",
           "exp/info-[inspect].py"].py"]

doc_url = "https://ncoop57.github.io/ds4se/"

git_url = "https://github.com/ncoop57/ds4se/tree/master/"

def custom_doc_links(name): return None
