# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/2.6_repr.word2vec.eval.ipynb (unless otherwise specified).

__all__ = []

# Cell
# Imports
import numpy as np
import gensim
import pandas as pd
from itertools import product
from random import sample
import functools

# Cell
#Visualizing wordEmbedding
from sklearn.decomposition import IncrementalPCA    # inital reduction
from sklearn.manifold import TSNE                   # final reduction
import numpy as np                                  # array handling