# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/2.3_repr.word2vec.train.ipynb (unless otherwise specified).

__all__ = []

# Cell
# Imports
import numpy as np
import pandas as pd

from pathlib import Path

from gensim.test.utils import common_texts, get_tmpfile
from gensim.models import Word2Vec